import os
import logging
from logging.handlers import RotatingFileHandler
from flask import Flask, jsonify, redirect, render_template, request
from flask_sqlalchemy import SQLAlchemy
from flask_migrate import Migrate
from flask_jwt_extended import JWTManager
from flask_swagger_ui import get_swaggerui_blueprint
from .config import Config
from flask_mail import Mail
from itsdangerous import URLSafeTimedSerializer
from flasgger import Swagger

db = SQLAlchemy()
migrate = Migrate()
jwt = JWTManager()
mail = Mail()
serializer = URLSafeTimedSerializer(Config.SECRET_KEY)

def create_app():
    app = Flask(__name__, static_folder='../static', template_folder='templates')
    app.config.from_object(Config)

    db.init_app(app)
    migrate.init_app(app, db)
    jwt.init_app(app)
    mail.init_app(app)

    if not os.path.exists('logs'):
        os.mkdir('logs')

    file_handler = RotatingFileHandler(app.config['LOG_FILE'], maxBytes=10240, backupCount=10)
    file_handler.setFormatter(logging.Formatter(
        '%(asctime)s %(levelname)s: %(message)s [in %(pathname)s:%(lineno)d]'
    ))
    file_handler.setLevel(logging.INFO)
    app.logger.addHandler(file_handler)

    app.logger.setLevel(logging.INFO)
    app.logger.info(f'{Config.APP_NAME} startup')

    from app.routes.auth import auth_bp
    from app.routes.user import user_bp
    from app.routes.email import email_bp
    from app.routes.tester import tester
    from app.routes.registerapp import registerapp

    app.register_blueprint(auth_bp, url_prefix='/auth')
    app.register_blueprint(user_bp, url_prefix='/user')
    app.register_blueprint(email_bp, url_prefix='/email')

    app.register_blueprint(tester)
    app.register_blueprint(registerapp)

    swagger = Swagger(app, config={
        "headers": [
            ('Access-Control-Allow-Origin', '*')
        ],
        "title": Config.APP_NAME,
        'specs': [
            {
                'endpoint': 'apispec_1',
                'route': '/apispec_1.json',
                'rule_filter': lambda rule: True,
                'model_filter': lambda tag: True,
            }
        ],
        "host": f"localhost:{app.config['PORT']}",
        'static_url_path': '/flasgger_static',
        "swagger_ui": True,
        'specs_route': '/apidoc/',
        'info': {
            'title': app.config['SWAGGER_TITLE'],
            'description': app.config['SWAGGER_DESCRIPTION'],
            'version': app.config['SWAGGER_VERSION']
        },
    }, template_file='../static/swagger.config.json')
    
    @app.route("/")
    def home():
        return redirect('/apidoc')

    @app.route('/main')
    def main():
        return render_template('index.html', app_name="Wheeeee!", my_list=[0,1,2,3,4,5])

    # Penanganan kesalahan
    @app.errorhandler(404)
    def not_found_error(error):
        app.logger.error(f'Not Found: {request.url}')
        return jsonify({"code": 404, "message": "Resource not found", "data": None}), 404

    @app.errorhandler(500)
    def internal_error(error):
        app.logger.error(f'Server Error: {error}, Path: {request.path}')
        db.session.rollback()
        return jsonify({"code": 500, "message": "Internal server error", "data": None}), 500

    @app.cli.command('seed_db')
    def seed_db_command():
        from app.seeder import seed_users
        seed_users()

    return app
